# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""The user interface for the Ubuntu One Installer."""

import subprocess

import gettext

from PyQt4 import QtGui, QtCore

from ubuntu_sso.credentials import Credentials

from ubuntu_sso.qt.gui import (
    EmailVerificationPage,
    ErrorPage,
    ForgottenPasswordPage,
    ResetPasswordPage,
    TosPage,
    SetupAccountPage,
)
from ubuntu_sso.qt.choose_sign_in_ui import \
    Ui_ChooseSignInPage
from ubuntu_sso.qt.current_user_sign_in_ui import \
    Ui_CurrentUserSignInPage
from ubuntu_sso.qt.email_verification_ui import \
    Ui_EmailVerificationPage
from ubuntu_sso.qt.error_message_ui import Ui_ErrorPage
from ubuntu_sso.qt.setup_account_ui import Ui_SetUpAccountPage
from ubuntu_sso.qt.terms_and_conditions_ui import Ui_TosPage
from ubuntu_sso.qt.success_message_ui import Ui_SuccessPage
from ubuntu_sso.qt.forgotten_password_ui import \
    Ui_ForgottenPasswordPage
from ubuntu_sso.qt.reset_password_ui import Ui_ResetPasswordPage
from ubuntu_sso.qt.controllers import (
    ChooseSignInController,
    CurrentUserController,
    EmailVerificationController,
    ErrorController,
    ForgottenPasswordController,
    ResetPasswordController,
    SetUpAccountController,
    SuccessController,
    TosController,
)

from ubuntuone.credentials import (
    APP_NAME,
    TC_URL,
    PING_URL,
    DESCRIPTION,
)

from ubuntuone_installer.logger import setup_logging
from ubuntuone_installer.gui.qt.ui import (
    license_ui,
    congratulations_ui,
)
from ubuntuone_installer.gui.qt.local_folders import LocalFoldersPage
from ubuntuone_installer.gui.qt.sync_now_or_later import SyncNowOrLaterPage

_ = gettext.gettext
SIGN_IN = _("Sign In")

# Invalid name logger
# pylint: disable=C0103
logger = setup_logging('qt.gui')
# pylint: enable=C0103


class LicensePage(QtGui.QWizardPage):
    """Wizard Page that displays the license info and links to the GPL."""

    def __init__(self, parent=None):
        super(LicensePage, self).__init__(parent)
        self.ui = license_ui.Ui_Form()
        self.ui.setupUi(self)
        self.setCommitPage(False)

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # Set the right texts and connections for buttons
        self.setButtonText(QtGui.QWizard.NextButton, _("Agree && Install"))
        self.setButtonText(QtGui.QWizard.CancelButton,
            _("Disagree && Cancel"))
        self.setButtonText(QtGui.QWizard.CustomButton1, _("&Print"))

        # This is just to catch an exception thrown when nothing
        # is connected to the signal. It's not an exceptional
        # condition at all.
        try:
            self.wizard().customButtonClicked.disconnect()
        except TypeError:
            pass

        self.wizard().customButtonClicked.connect(self.print_document)

        self.wizard().setButtonLayout([
            QtGui.QWizard.CustomButton1,
            QtGui.QWizard.Stretch,
            QtGui.QWizard.CancelButton,
            QtGui.QWizard.NextButton,
            QtGui.QWizard.FinishButton,
            ])

    def print_document(self, button_id):
        """Print the document displayed in textBrowser."""
        if button_id == QtGui.QWizard.CustomButton1:
            document = self.ui.textBrowser.document()
            printer = QtGui.QPrinter(QtGui.QPrinter.HighResolution)

            dialog = QtGui.QPrintDialog(printer, self)

            dialog.setWindowTitle(_("Send to printer"))
            if dialog.exec_() != QtGui.QDialog.Accepted:
                return
            printer.setFullPage(True)
            printer.setPageSize(QtGui.QPrinter.A4)
            document.print_(printer)


class SignInPage(QtGui.QWizardPage):
    """Wizard Page that lets the user Sign into Ubuntu One."""

    def __init__(self, ui=None, controller=None, parent=None):
        super(SignInPage, self).__init__(parent)
        self.ui = ui
        self.ui.setupUi(self)
        self.controller = controller
        self.controller.setupUi(self)
        self.next = -1

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        self.setButtonText(QtGui.QWizard.CancelButton,
            _("Close window and setup later"))
        # Layout without custom button 1,
        # without finish button
        self.wizard().setButtonLayout([
            QtGui.QWizard.CancelButton,
            QtGui.QWizard.Stretch,
            QtGui.QWizard.NextButton,
            ])

    def nextId(self):
        """Provide the next id."""
        return self.next


class CurrentUserSignInPage(QtGui.QWizardPage):
    """Wizard Page that lets a current user Sign into Ubuntu One."""

    def __init__(self, ui=None, controller=None, parent=None):
        super(CurrentUserSignInPage, self).__init__(parent)
        self.ui = ui
        self.ui.setupUi(self)
        self.controller = controller
        self.controller.setupUi(self)
        self.next = -1

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def nextId(self):
        """Provide the next id."""
        return self.next


class SuccessPage(QtGui.QWizardPage):
    """Shown after SSO login, before setup."""

    def __init__(self, ui=None, controller=None, parent=None):
        super(SuccessPage, self).__init__(parent)
        self.ui = ui
        self.ui.setupUi(self)
        self.controller = controller
        self.controller.setupUi(self)
        self.next = -1

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.NextButton,
            QtGui.QWizard.FinishButton])
        self.wizard().setButtonText(QtGui.QWizard.FinishButton,
            _("Skip tour, go to Dashboard"))

    def nextId(self):
        """Provide the next id."""
        return self.next


class CongratulationsPage(QtGui.QWizardPage):
    """Final page of the wizard."""

    def __init__(self, parent=None):
        super(CongratulationsPage, self).__init__(parent)
        self.ui = congratulations_ui.Ui_Form()
        self.ui.setupUi(self)
        self.setFinalPage(True)

    # Invalid names of Qt-inherited methods
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # We need custom buttons
        self.wizard().setButtonText(QtGui.QWizard.FinishButton,
            _("Go to my Ubuntu One dashboard"))
        # Layout without custom button 1,
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.FinishButton,
            ])
        if self.field("sync_now").toBool() \
            or self.field("sync_custom").toBool():
            self.ui.progressContainer.setVisible(True)
        else:
            self.ui.progressContainer.setVisible(False)


class MainWindow(QtGui.QWizard):
    """The Main Window of the Installer wizard."""

    # Invalid constant names and Qt-inherited methods
    # pylint: disable=C0103

    LICENSE_PAGE = 0
    SIGNIN_PAGE = 1
    SYNC_NOW_OR_LATER_PAGE = 2
    CONGRATULATIONS_PAGE = 3

    # definition of the signals raised by the widget (for SSO)
    recoverableError = QtCore.pyqtSignal('QString', 'QString')
    loginSuccess = QtCore.pyqtSignal('QString', 'QString')
    registrationSuccess = QtCore.pyqtSignal('QString', 'QString')

    def __init__(self, close_callback=None):
        """Initialize this instance."""
        # Used to decide the next page dynamically
        self._next_id = None

        # Data for SSO
        self.app_name = APP_NAME
        self.help_text = DESCRIPTION
        self.tc_url = TC_URL
        self.ping_url = PING_URL

        super(MainWindow, self).__init__()
        self.close_callback = close_callback

        self.creds = Credentials(
            app_name=self.app_name,
            ui_module="ubuntuone_installer.gui.qt.embedded_sso",
            tc_url=self.tc_url,
            ping_url=self.ping_url,
            )

        self.setOption(self.NoBackButtonOnStartPage, True)
        self.setOption(self.DisabledBackButtonOnLastPage, True)
        # PyQt doesn't support the (int, page) version of addPage
        # Add the pages in the right order

        # pylint: disable=C0103
        self.LICENSE_PAGE = self.addPage(LicensePage())

        #SSO Pages

        self.sign_in_controller = ChooseSignInController(title=SIGN_IN)
        self.sign_in_page = SignInPage(ui=Ui_ChooseSignInPage(),
                                       controller=self.sign_in_controller,
                                       parent=self)
        self.setup_controller = SetUpAccountController()
        self.setup_account = SetupAccountPage(Ui_SetUpAccountPage(),
                                              self.setup_controller,
                                              parent=self)
        self.tos = TosPage(Ui_TosPage(),
            TosController(tos_url=TC_URL),
            parent=self)
        self.email_verification = EmailVerificationPage(
                                                Ui_EmailVerificationPage(),
                                                EmailVerificationController())
        self.current_user_controller = CurrentUserController(SIGN_IN)
        self.current_user = CurrentUserSignInPage(Ui_CurrentUserSignInPage(),
                                                  self.current_user_controller,
                                                  parent=self)
        self.success_controller = SuccessController()
        self.success = SuccessPage(Ui_SuccessPage(), self.success_controller,
                                   parent=self)
        self.error_controller = ErrorController()
        self.error = ErrorPage(Ui_ErrorPage(), self.error_controller)
        self.forgotten_pwd_controller = ForgottenPasswordController()
        self.forgotten = ForgottenPasswordPage(Ui_ForgottenPasswordPage(),
                                               self.forgotten_pwd_controller,
                                               parent=self)
        self.reset_password_controller = ResetPasswordController()
        self.reset_password = ResetPasswordPage(Ui_ResetPasswordPage(),
                                                self.reset_password_controller,
                                                parent=self)

        self.SIGNIN_PAGE = self.addPage(self.sign_in_page)
        self.setup_account_page_id = self.addPage(self.setup_account)
        self.tos_page_id = self.addPage(self.tos)
        self.email_verification_page_id = self.addPage(self.email_verification)
        self.current_user_page_id = self.addPage(self.current_user)
        self.SUCCESS_PAGE = self.addPage(self.success)
        self.ERROR_PAGE = self.addPage(self.error)
        self.forgotten_password_page_id = self.addPage(self.forgotten)
        self.reset_password_page_id = self.addPage(self.reset_password)

        self.loginSuccess.connect(self.login_success_slot)
        self.registrationSuccess.connect(self.registration_success_slot)

        # End of SSO pages

        self.local_folders_page = LocalFoldersPage()
        self.local_folders_page_id = self.addPage(self.local_folders_page)
        self.SYNC_NOW_OR_LATER_PAGE = self.addPage(SyncNowOrLaterPage())
        self.CONGRATULATIONS_PAGE = self.addPage(CongratulationsPage())

    # Invalid name "closeEvent"
    # pylint: disable=C0103

    def nextId(self):
        """Selects which page to display next."""
        if self._next_id is not None:
            return self._next_id
        else:
            return QtGui.QWizard.nextId(self)

    def login_success_slot(self):
        """Called on successful login."""
        self._next_id = self.SUCCESS_PAGE
        self.next()
        self._next_id = self.local_folders_page_id

    def registration_success_slot(self):
        """Called on successful registration."""
        self._next_id = self.SUCCESS_PAGE
        self.next()
        self._next_id = self.local_folders_page_id

    def done(self, result):
        """The main window is being closed, call any custom callback."""
        if result == 1:  # Finished, not cancelled
            subprocess.Popen(["ubuntuone-control-panel-qt", ])
        if self.close_callback is not None:
            self.close_callback()
        QtGui.QWizard.done(self, result)
